@@ -108,6 +108,8 @@ static void get_properties(DBusConnection *connection,
 		return;
 	}
 
+	dbus_message_set_auto_start(message, FALSE);
+
 	if (dbus_connection_send_with_reply(connection, message,
 						&call, TIMEOUT) == FALSE) {
 		connman_error("Failed to get properties for %s", interface);
@@ -236,6 +238,8 @@ static int pan_connect(struct connman_network *network)
 	if (message == NULL)
 		return -ENOMEM;
 
+	dbus_message_set_auto_start(message, FALSE);
+
 	dbus_message_append_args(message, DBUS_TYPE_STRING, &uuid,
 							DBUS_TYPE_INVALID);
 
@@ -310,6 +314,8 @@ static int pan_disconnect(struct connman_network *network)
 	if (message == NULL)
 		return -ENOMEM;
 
+	dbus_message_set_auto_start(message, FALSE);
+
 	dbus_message_append_args(message, DBUS_TYPE_INVALID);
 
 	if (dbus_connection_send_with_reply(data->connection, message,
@@ -400,6 +406,8 @@ static int change_powered(DBusConnection *connection, const char *path,
 	if (message == NULL)
 		return -ENOMEM;
 
+	dbus_message_set_auto_start(message, FALSE);
+
 	dbus_message_iter_init_append(message, &iter);
 	connman_dbus_property_append_variant(&iter, "Powered",
 						DBUS_TYPE_BOOLEAN, &powered);
@@ -650,7 +658,7 @@ static void adapter_properties(DBusConnection *connection, const char *path,
 			connman_device_set_index(adapter, index);
 	}
 
-	connman_device_set_interface(adapter, node);
+	connman_device_set_interface(adapter, node, NULL);
 
 	connman_device_set_mode(adapter, CONNMAN_DEVICE_MODE_NETWORK_MULTIPLE);
 
@@ -738,6 +746,8 @@ static void bluetooth_connect(DBusConnection *connection, void *user_data)
 	if (message == NULL)
 		return;
 
+	dbus_message_set_auto_start(message, FALSE);
+
 	if (dbus_connection_send_with_reply(connection, message,
 						&call, TIMEOUT) == FALSE) {
 		connman_error("Failed to get Bluetooth adapters");
