@@ -37,7 +37,10 @@ struct connman_device {
 	enum connman_device_type type;
 	enum connman_device_mode mode;
 	connman_bool_t secondary;
+	connman_bool_t offlinemode;
+	connman_bool_t blocked;
 	connman_bool_t powered;
+	connman_bool_t powered_persistent;
 	connman_bool_t carrier;
 	connman_bool_t scanning;
 	connman_bool_t disconnected;
@@ -48,6 +51,7 @@ struct connman_device {
 	char *interface;
 	char *control;
 	char *ident;
+	int phyindex;
 	unsigned int connections;
 	guint scan_timeout;
 	struct connman_ipconfig *ipconfig;
@@ -484,6 +488,10 @@ static DBusMessage *set_property(DBusConnection *conn,
 
 		dbus_message_iter_get_basic(&value, &powered);
 
+		device->powered_persistent = powered;
+
+		__connman_storage_save_device(device);
+
 		if (device->powered == powered)
 			return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
 
@@ -522,6 +530,8 @@ static DBusMessage *set_property(DBusConnection *conn,
 		if (device->scan_interval != interval) {
 			device->scan_interval = interval;
 
+			__connman_storage_save_device(device);
+
 			reset_scan_trigger(device);
 		}
 	} else if (g_str_has_prefix(name, "IPv4.") == TRUE) {
@@ -543,8 +553,6 @@ static DBusMessage *set_property(DBusConnection *conn,
 	} else
 		return __connman_error_invalid_property(msg);
 
-	__connman_storage_save_device(device);
-
 	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
 }
 
@@ -656,46 +664,6 @@ static void unregister_interface(struct connman_element *element)
 						CONNMAN_DEVICE_INTERFACE);
 }
 
-static void device_enable(struct connman_device *device)
-{
-	DBG("device %p", device);
-
-	if (device->powered == TRUE)
-		return;
-
-	if (device->driver->enable) {
-		if (device->driver->enable(device) == 0) {
-			enum connman_service_type type;
-
-			device->powered = TRUE;
-
-			type = __connman_device_get_service_type(device);
-			__connman_notifier_enable(type);
-		}
-	}
-}
-
-static void device_disable(struct connman_device *device)
-{
-	DBG("device %p", device);
-
-	if (device->powered == FALSE)
-		return;
-
-	g_hash_table_remove_all(device->networks);
-
-	if (device->driver->disable) {
-		if (device->driver->disable(device) == 0) {
-			enum connman_service_type type;
-
-			device->powered = FALSE;
-
-			type = __connman_device_get_service_type(device);
-			__connman_notifier_disable(type);
-		}
-	}
-}
-
 static int setup_device(struct connman_device *device)
 {
 	int err;
@@ -721,7 +689,9 @@ static int setup_device(struct connman_device *device)
 		break;
 	}
 
-	device_enable(device);
+	if (device->offlinemode == FALSE &&
+				device->powered_persistent == TRUE)
+		__connman_device_enable(device);
 
 	return 0;
 }
@@ -753,7 +723,7 @@ static void remove_device(struct connman_device *device)
 {
 	DBG("device %p", device);
 
-	device_disable(device);
+	__connman_device_disable(device);
 
 	switch (device->mode) {
 	case CONNMAN_DEVICE_MODE_UNKNOWN:
@@ -927,6 +897,10 @@ struct connman_device *connman_device_create(const char *node,
 	device->mode      = CONNMAN_DEVICE_MODE_UNKNOWN;
 	device->secondary = FALSE;
 
+	device->powered_persistent = TRUE;
+
+	device->phyindex = -1;
+
 	switch (type) {
 	case CONNMAN_DEVICE_TYPE_UNKNOWN:
 	case CONNMAN_DEVICE_TYPE_VENDOR:
@@ -1052,6 +1026,17 @@ int connman_device_get_index(struct connman_device *device)
 	return device->element.index;
 }
 
+int __connman_device_get_phyindex(struct connman_device *device)
+{
+	return device->phyindex;
+}
+
+void __connman_device_set_phyindex(struct connman_device *device,
+							int phyindex)
+{
+	device->phyindex = phyindex;
+}
+
 /**
  * connman_device_set_interface:
  * @device: device structure
@@ -1207,6 +1192,29 @@ int connman_device_set_powered(struct connman_device *device,
 	return 0;
 }
 
+int __connman_device_set_blocked(struct connman_device *device,
+						connman_bool_t blocked)
+{
+	connman_bool_t powered;
+
+	DBG("device %p blocked %d", device, blocked);
+
+	if (device->blocked == blocked)
+		return -EALREADY;
+
+	device->blocked = blocked;
+
+	if (device->offlinemode == TRUE)
+		return 0;
+
+	if (blocked == FALSE)
+		powered = device->powered_persistent;
+	else
+		powered = FALSE;
+
+	return set_powered(device, powered);
+}
+
 /**
  * connman_device_set_carrier:
  * @device: device structure
@@ -1249,30 +1257,78 @@ int __connman_device_scan(struct connman_device *device)
 
 int __connman_device_enable(struct connman_device *device)
 {
+	enum connman_service_type type;
+	int err;
+
+	DBG("device %p", device);
+
 	if (!device->driver || !device->driver->enable)
 		return -EOPNOTSUPP;
 
 	if (device->powered == TRUE)
 		return -EALREADY;
 
-	device_enable(device);
+	err = device->driver->enable(device);
+	if (err < 0)
+		return err;
+
+	device->powered = TRUE;
+
+	type = __connman_device_get_service_type(device);
+	__connman_notifier_enable(type);
 
 	return 0;
 }
 
+int __connman_device_enable_persistent(struct connman_device *device)
+{
+	DBG("device %p", device);
+
+	device->powered_persistent = TRUE;
+
+	__connman_storage_save_device(device);
+
+	return __connman_device_enable(device);
+}
+
 int __connman_device_disable(struct connman_device *device)
 {
+	enum connman_service_type type;
+	int err;
+
+	DBG("device %p", device);
+
 	if (!device->driver || !device->driver->disable)
 		return -EOPNOTSUPP;
 
 	if (device->powered == FALSE)
 		return -ENOLINK;
 
-	device_disable(device);
+	g_hash_table_remove_all(device->networks);
+
+	err = device->driver->disable(device);
+	if (err < 0)
+		return err;
+
+	device->powered = FALSE;
+
+	type = __connman_device_get_service_type(device);
+	__connman_notifier_disable(type);
 
 	return 0;
 }
 
+int __connman_device_disable_persistent(struct connman_device *device)
+{
+	DBG("device %p", device);
+
+	device->powered_persistent = FALSE;
+
+	__connman_storage_save_device(device);
+
+	return __connman_device_disable(device);
+}
+
 int __connman_device_connect(struct connman_device *device)
 {
 	DBG("device %p", device);
@@ -1529,11 +1585,16 @@ static void set_offlinemode(struct connman_element *element, gpointer user_data)
 	if (device == NULL)
 		return;
 
+	device->offlinemode = offlinemode;
+
 	powered = (offlinemode == TRUE) ? FALSE : TRUE;
 
 	if (device->powered == powered)
 		return;
 
+	if (device->powered_persistent == FALSE)
+		powered = FALSE;
+
 	set_powered(device, powered);
 }
 
@@ -1544,7 +1605,7 @@ int __connman_device_set_offlinemode(connman_bool_t offlinemode)
 	__connman_element_foreach(NULL, CONNMAN_ELEMENT_TYPE_DEVICE,
 			set_offlinemode, GUINT_TO_POINTER(offlinemode));
 
-	__connman_notifier_offline_mode(offlinemode);
+	__connman_notifier_offlinemode(offlinemode);
 
 	return 0;
 }
@@ -1595,8 +1656,6 @@ int connman_device_add_network(struct connman_device *device,
 
 	__connman_network_set_device(network, device);
 
-	__connman_storage_load_network(network);
-
 	err = connman_element_register((struct connman_element *) network,
 							&device->element);
 	if (err < 0) {
@@ -1680,15 +1739,7 @@ int connman_device_register(struct connman_device *device)
 
 	__connman_storage_load_device(device);
 
-	switch (device->mode) {
-	case CONNMAN_DEVICE_MODE_UNKNOWN:
-	case CONNMAN_DEVICE_MODE_TRANSPORT_IP:
-		break;
-	case CONNMAN_DEVICE_MODE_NETWORK_SINGLE:
-	case CONNMAN_DEVICE_MODE_NETWORK_MULTIPLE:
-		__connman_storage_init_network(device);
-		break;
-	}
+	device->offlinemode = __connman_manager_get_offlinemode();
 
 	type = __connman_device_get_service_type(device);
 	__connman_notifier_register(type);
@@ -1806,38 +1857,27 @@ static struct connman_driver device_driver = {
 static int device_load(struct connman_device *device)
 {
 	GKeyFile *keyfile;
-	gchar *pathname, *identifier, *data = NULL;
-	gsize length;
+	GError *error = NULL;
+	gchar *identifier;
+	connman_bool_t powered;
 	int val;
 
 	DBG("device %p", device);
 
-	pathname = g_strdup_printf("%s/%s.conf", STORAGEDIR,
-					__connman_profile_active_ident());
-	if (pathname == NULL)
-		return -ENOMEM;
-
-	keyfile = g_key_file_new();
-
-	if (g_file_get_contents(pathname, &data, &length, NULL) == FALSE) {
-		g_free(pathname);
-		return -ENOENT;
-	}
-
-	g_free(pathname);
-
-	if (g_key_file_load_from_data(keyfile, data, length,
-							0, NULL) == FALSE) {
-		g_free(data);
-		return -EILSEQ;
-	}
-
-	g_free(data);
+	keyfile = __connman_storage_open();
+	if (keyfile == NULL)
+		return 0;
 
 	identifier = g_strdup_printf("device_%s", device->element.name);
 	if (identifier == NULL)
 		goto done;
 
+	powered = g_key_file_get_boolean(keyfile, identifier,
+						"Powered", &error);
+	if (error == NULL)
+		device->powered_persistent = powered;
+	g_clear_error(&error);
+
 	switch (device->mode) {
 	case CONNMAN_DEVICE_MODE_UNKNOWN:
 	case CONNMAN_DEVICE_MODE_TRANSPORT_IP:
@@ -1845,51 +1885,39 @@ static int device_load(struct connman_device *device)
 	case CONNMAN_DEVICE_MODE_NETWORK_SINGLE:
 	case CONNMAN_DEVICE_MODE_NETWORK_MULTIPLE:
 		val = g_key_file_get_integer(keyfile, identifier,
-							"ScanInterval", NULL);
-		if (val > 0)
+						"ScanInterval", &error);
+		if (error == NULL && val > 0)
 			device->scan_interval = val;
+		g_clear_error(&error);
 		break;
 	}
 
 done:
-	g_key_file_free(keyfile);
-
 	g_free(identifier);
 
+	__connman_storage_close(keyfile, FALSE);
+
 	return 0;
 }
 
 static int device_save(struct connman_device *device)
 {
 	GKeyFile *keyfile;
-	gchar *pathname, *identifier = NULL, *data = NULL;
-	gsize length;
+	gchar *identifier;
 
 	DBG("device %p", device);
 
-	pathname = g_strdup_printf("%s/%s.conf", STORAGEDIR,
-					__connman_profile_active_ident());
-	if (pathname == NULL)
-		return -ENOMEM;
+	keyfile = __connman_storage_open();
+	if (keyfile == NULL)
+		return 0;
 
-	keyfile = g_key_file_new();
-
-	if (g_file_get_contents(pathname, &data, &length, NULL) == FALSE)
-		goto update;
-
-	if (length > 0) {
-		if (g_key_file_load_from_data(keyfile, data, length,
-							0, NULL) == FALSE)
-			goto done;
-	}
-
-	g_free(data);
-
-update:
 	identifier = g_strdup_printf("device_%s", device->element.name);
 	if (identifier == NULL)
 		goto done;
 
+	g_key_file_set_boolean(keyfile, identifier,
+					"Powered", device->powered_persistent);
+
 	switch (device->mode) {
 	case CONNMAN_DEVICE_MODE_UNKNOWN:
 	case CONNMAN_DEVICE_MODE_TRANSPORT_IP:
@@ -1902,18 +1930,10 @@ update:
 		break;
 	}
 
-	data = g_key_file_to_data(keyfile, &length, NULL);
-
-	if (g_file_set_contents(pathname, data, length, NULL) == FALSE)
-		connman_error("Failed to store device information");
-
 done:
-	g_free(data);
-
-	g_key_file_free(keyfile);
-
 	g_free(identifier);
-	g_free(pathname);
+
+	__connman_storage_close(keyfile, TRUE);
 
 	return 0;
 }
