@@ -274,6 +274,7 @@ struct connman_service *__connman_element_get_service(struct connman_element *el
 	case CONNMAN_DEVICE_TYPE_VENDOR:
 	case CONNMAN_DEVICE_TYPE_BLUETOOTH:
 	case CONNMAN_DEVICE_TYPE_GPS:
+	case CONNMAN_DEVICE_TYPE_MBM:
 	case CONNMAN_DEVICE_TYPE_HSO:
 	case CONNMAN_DEVICE_TYPE_NOZOMI:
 	case CONNMAN_DEVICE_TYPE_HUAWEI:
@@ -329,6 +330,187 @@ const char *__connman_element_get_network_path(struct connman_element *element)
 	return __connman_element_get_network_path(element->parent);
 }
 
+struct find_data {
+	enum connman_device_type type;
+	struct connman_device *device;
+};
+
+static gboolean find_device(GNode *node, gpointer user_data)
+{
+	struct connman_element *element = node->data;
+	struct find_data *data = user_data;
+
+	if (element->type != CONNMAN_ELEMENT_TYPE_DEVICE)
+		return FALSE;
+
+	if (element->device == NULL)
+		return FALSE;
+
+	if (data->type != connman_device_get_type(element->device))
+		return FALSE;
+
+	data->device = element->device;
+
+	return TRUE;
+}
+
+struct connman_device *__connman_element_find_device(enum connman_device_type type)
+{
+	struct find_data data = { .type = type, .device = NULL };
+
+	g_node_traverse(element_root, G_PRE_ORDER,
+				G_TRAVERSE_ALL, -1, find_device, &data);
+
+	return data.device;
+}
+
+static gboolean request_scan(GNode *node, gpointer user_data)
+{
+	struct connman_element *element = node->data;
+	struct find_data *data = user_data;
+	enum connman_device_type type;
+
+	if (element->type != CONNMAN_ELEMENT_TYPE_DEVICE)
+		return FALSE;
+
+	if (element->device == NULL)
+		return FALSE;
+
+	type = connman_device_get_type(element->device);
+
+	switch (type) {
+	case CONNMAN_DEVICE_TYPE_UNKNOWN:
+	case CONNMAN_DEVICE_TYPE_VENDOR:
+	case CONNMAN_DEVICE_TYPE_ETHERNET:
+	case CONNMAN_DEVICE_TYPE_BLUETOOTH:
+	case CONNMAN_DEVICE_TYPE_GPS:
+	case CONNMAN_DEVICE_TYPE_MBM:
+	case CONNMAN_DEVICE_TYPE_HSO:
+	case CONNMAN_DEVICE_TYPE_NOZOMI:
+	case CONNMAN_DEVICE_TYPE_HUAWEI:
+	case CONNMAN_DEVICE_TYPE_NOVATEL:
+		return FALSE;
+	case CONNMAN_DEVICE_TYPE_WIFI:
+	case CONNMAN_DEVICE_TYPE_WIMAX:
+		if (data->type != CONNMAN_DEVICE_TYPE_UNKNOWN &&
+							data->type != type)
+			return FALSE;
+		break;
+	}
+
+	__connman_device_scan(element->device);
+
+	return FALSE;
+}
+
+int __connman_element_request_scan(enum connman_device_type type)
+{
+	struct find_data data = { .type = type, .device = NULL };
+
+	g_node_traverse(element_root, G_PRE_ORDER,
+				G_TRAVERSE_ALL, -1, request_scan, &data);
+
+	return 0;
+}
+
+static gboolean enable_technology(GNode *node, gpointer user_data)
+{
+	struct connman_element *element = node->data;
+	struct find_data *data = user_data;
+	enum connman_device_type type;
+
+	if (element->type != CONNMAN_ELEMENT_TYPE_DEVICE)
+		return FALSE;
+
+	if (element->device == NULL)
+		return FALSE;
+
+	type = connman_device_get_type(element->device);
+
+	switch (type) {
+	case CONNMAN_DEVICE_TYPE_UNKNOWN:
+	case CONNMAN_DEVICE_TYPE_VENDOR:
+	case CONNMAN_DEVICE_TYPE_MBM:
+	case CONNMAN_DEVICE_TYPE_HSO:
+	case CONNMAN_DEVICE_TYPE_NOZOMI:
+	case CONNMAN_DEVICE_TYPE_HUAWEI:
+	case CONNMAN_DEVICE_TYPE_NOVATEL:
+		return FALSE;
+	case CONNMAN_DEVICE_TYPE_ETHERNET:
+	case CONNMAN_DEVICE_TYPE_WIFI:
+	case CONNMAN_DEVICE_TYPE_WIMAX:
+	case CONNMAN_DEVICE_TYPE_BLUETOOTH:
+	case CONNMAN_DEVICE_TYPE_GPS:
+		if (data->type != CONNMAN_DEVICE_TYPE_UNKNOWN &&
+							data->type != type)
+			return FALSE;
+		break;
+	}
+
+	__connman_device_enable_persistent(element->device);
+
+	return FALSE;
+}
+
+int __connman_element_enable_technology(enum connman_device_type type)
+{
+	struct find_data data = { .type = type, .device = NULL };
+
+	g_node_traverse(element_root, G_PRE_ORDER,
+				G_TRAVERSE_ALL, -1, enable_technology, &data);
+
+	return 0;
+}
+
+static gboolean disable_technology(GNode *node, gpointer user_data)
+{
+	struct connman_element *element = node->data;
+	struct find_data *data = user_data;
+	enum connman_device_type type;
+
+	if (element->type != CONNMAN_ELEMENT_TYPE_DEVICE)
+		return FALSE;
+
+	if (element->device == NULL)
+		return FALSE;
+
+	type = connman_device_get_type(element->device);
+
+	switch (type) {
+	case CONNMAN_DEVICE_TYPE_UNKNOWN:
+	case CONNMAN_DEVICE_TYPE_VENDOR:
+	case CONNMAN_DEVICE_TYPE_MBM:
+	case CONNMAN_DEVICE_TYPE_HSO:
+	case CONNMAN_DEVICE_TYPE_NOZOMI:
+	case CONNMAN_DEVICE_TYPE_HUAWEI:
+	case CONNMAN_DEVICE_TYPE_NOVATEL:
+		return FALSE;
+	case CONNMAN_DEVICE_TYPE_ETHERNET:
+	case CONNMAN_DEVICE_TYPE_WIFI:
+	case CONNMAN_DEVICE_TYPE_WIMAX:
+	case CONNMAN_DEVICE_TYPE_BLUETOOTH:
+	case CONNMAN_DEVICE_TYPE_GPS:
+		if (data->type != CONNMAN_DEVICE_TYPE_UNKNOWN &&
+							data->type != type)
+			return FALSE;
+		break;
+	}
+
+	__connman_device_disable_persistent(element->device);
+
+	return FALSE;
+}
+
+int __connman_element_disable_technology(enum connman_device_type type)
+{
+	struct find_data data = { .type = type, .device = NULL };
+
+	g_node_traverse(element_root, G_PRE_ORDER,
+				G_TRAVERSE_ALL, -1, disable_technology, &data);
+
+	return 0;
+}
+
 static gint compare_priority(gconstpointer a, gconstpointer b)
 {
 	const struct connman_driver *driver1 = a;
@@ -773,7 +955,7 @@ static gboolean get_static_array_property(struct connman_element *element,
 
 	property = g_hash_table_lookup(element->properties, name);
 	if (property != NULL) {
-		*((char **) value) = property->value;
+		*((void **) value) = property->value;
 		*len = property->size;
 		found = TRUE;
 	}
@@ -1145,9 +1327,11 @@ static void register_element(gpointer data, gpointer user_data)
 	if (element->type == CONNMAN_ELEMENT_TYPE_DHCP) {
 		element->parent->configuring = TRUE;
 
+#if 0
 		if (__connman_element_count(NULL,
 					CONNMAN_ELEMENT_TYPE_CONNECTION) == 0)
 			emit_state_change(connection, "connecting");
+#endif
 	}
 
 	if (element->type == CONNMAN_ELEMENT_TYPE_CONNECTION) {
@@ -1335,6 +1519,8 @@ static enum connman_service_error convert_error(enum connman_element_error error
 		break;
 	case CONNMAN_ELEMENT_ERROR_DHCP_FAILED:
 		return CONNMAN_SERVICE_ERROR_DHCP_FAILED;
+	case CONNMAN_ELEMENT_ERROR_CONNECT_FAILED:
+		return CONNMAN_SERVICE_ERROR_CONNECT_FAILED;
 	}
 
 	return CONNMAN_SERVICE_ERROR_UNKNOWN;
@@ -1416,16 +1602,18 @@ void __connman_element_start(void)
 {
 	DBG("");
 
+	__connman_storage_load_global();
+
 	g_node_traverse(element_root, G_PRE_ORDER, G_TRAVERSE_ALL, -1,
 							probe_node, NULL);
 
 	started = TRUE;
 
-	__connman_storage_init_device();
+	__connman_rtnl_init();
+	__connman_udev_init();
 
 	__connman_connection_init();
 	__connman_ipv4_init();
-	__connman_detect_init();
 	__connman_rfkill_init();
 }
 
@@ -1434,9 +1622,10 @@ void __connman_element_stop(void)
 	DBG("");
 
 	__connman_rfkill_cleanup();
-	__connman_detect_cleanup();
 	__connman_ipv4_cleanup();
 	__connman_connection_cleanup();
+
+	__connman_storage_save_global();
 }
 
 static gboolean free_driver(GNode *node, gpointer data)
